#include <QCoreApplication>
#include <QApplication>
#include <QDebug>
#include <QTimer>
#include <QFileInfo>

#ifdef __linux__
#include <stdlib.h> // system()
#endif

#include "message_handler.h"
#include "plugins/interfaces.h"

#include <unistd.h>
#include <thread>
#include <memory>
#include <QSharedMemory>
#include <QRunnable>
#include <QThreadPool>
#include <QDir>
#include <QProcess>
#include <QCommandLineParser>
#include <QStandardPaths>

#include "update.h"
#include "worker_thread.h"
#include "worker.h"

#include <thread>

#ifdef PTU5
#define SERIAL_PORT "ttymxc2"
#else
#define SERIAL_PORT "ttyUSB0"
#endif

class hwinf;
static void doWork(hwinf *hw, QString update_ctrl_file,
                   QString workingDir, bool maintenanceMode) {
    std::this_thread::sleep_for(std::chrono::milliseconds(2000));
    Update update(hw, update_ctrl_file, workingDir, maintenanceMode);
    update.doUpdate();
    std::this_thread::sleep_for(std::chrono::milliseconds(2000));
    QCoreApplication::quit();
}

// argv[1]: file to send to dc
int main(int argc, char *argv[]) {

    QByteArray const value = qgetenv("XDG_RUNTIME_DIR");
    if (value.size() == 0) {
        qputenv("XDG_RUNTIME_DIR", "/run/user/0");
    }

    QApplication a(argc, argv);
    QApplication::setApplicationName("ATBUpdateTool");
    QApplication::setApplicationVersion("1.0");

    if (!messageHandlerInstalled()) { // change internal qt-QDebug-handling
        atbInstallMessageHandler(atbDebugOutput);
        setDebugLevel(QtMsgType::QtDebugMsg);
        //setDebugLevel(QtMsgType::QtDebugMsg);
    }

    QCommandLineParser parser;
    parser.setApplicationDescription("Download tool for downloading device controller firmware, printer json-files and executing opkg-commands.");
    parser.addHelpOption();
    parser.addVersionOption();

    QCommandLineOption pluginDirectoryOption(QStringList() << "plugin-directory" << "plugin-directory",
        QCoreApplication::translate("main", "Where to find dc-plugin."),
        QCoreApplication::translate("main", "directory"));
    QString const pluginDefault = "./plugins";
    pluginDirectoryOption.setDefaultValue(pluginDefault);
    parser.addOption(pluginDirectoryOption);

    QCommandLineOption pluginNameOption(QStringList() << "plugin-name" << "plugin-name",
        QCoreApplication::translate("main", "Name of dc-plugin."),
        QCoreApplication::translate("main", "directory"));
    QString const pluginNameDefault = "libCAmaster.so";
    pluginNameOption.setDefaultValue(pluginNameDefault);
    parser.addOption(pluginNameOption);

    QCommandLineOption workingDirectoryOption(QStringList() << "working-directory" << "working-directory",
        QCoreApplication::translate("main", "working directory of update-script."),
        QCoreApplication::translate("main", "directory"));
    QString const workingDirectoryDefault = ".";
    workingDirectoryOption.setDefaultValue(workingDirectoryDefault);
    parser.addOption(workingDirectoryOption);

    QCommandLineOption maintenanceOption("m",
        QCoreApplication::translate("main", "Maintenance mode for underlying script"));
    parser.addOption(maintenanceOption);

    // TODO:
    // add some additional parameters
    // --dry-run
    // -d: only update device-controller firmware
    // -j: only update json-files
    // -o: only execute opkg-commnds

    // Process the actual command line arguments given by the user
    parser.process(a);
    QString plugInDir = parser.value(pluginDirectoryOption);
    QString plugInName = parser.value(pluginNameOption);
    QString workingDir = parser.value(workingDirectoryOption);
    bool maintenanceMode = parser.isSet(maintenanceOption);
    QString const rtPath = QCoreApplication::applicationDirPath();

    if (plugInDir == pluginDefault) {
        plugInDir = (rtPath + "/" + pluginDefault);
    }
    if (!QDir(plugInDir).exists()) {
        qCritical() << plugInDir
                    << "does not exists, but has to contain dc-library";
        exit(-1);
    }
    qInfo() << "pwd" << "=" << rtPath;
    qInfo() << "plugInDir" << "=" << plugInDir;
    qInfo() << "plugInName" << "=" << plugInName;
    qInfo() << "workingDir" << "=" << workingDir;
    qInfo() << "maintenanceMode" << "=" << maintenanceMode;

    // before loading the library, delete all possible shared memory segments
#if defined Q_OS_LINUX || defined Q_OS_UNIX
    // system("rm -rf /tmp/qipc*");
#else
#error "Only tested under UNIX/LINUX"
#endif

    hwinf *hw = Update::loadDCPlugin(QDir(plugInDir), plugInName);
    hw->dc_autoRequest(false);

    QString const update_ctrl_file = "/opt/app/tools/atbupdate/update_log.csv";
    std::thread t(doWork, hw, update_ctrl_file, workingDir, maintenanceMode);

    int ret = a.exec();
    t.join();

    return ret;
}
